@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -24,6 +24,7 @@
 #include "gva-error.h"
 #include "gva-favorites.h"
 #include "gva-mame.h"
+#include "gva-nplayers.h"
 #include "gva-util.h"
 
 #define ASSERT_OK(code) \
@@ -32,6 +33,10 @@
 /* Based on MAME's DTD */
 #define MAX_ELEMENT_DEPTH 4
 
+/* The new <dipswitch> and <configuration> attributes in 0.136 are
+ * REQUIRED, but we are leaving them as optional in the table schema
+ * for backward compatibility with older MAME versions. */
+
 #define SQL_CREATE_TABLE_MAME \
         "CREATE TABLE IF NOT EXISTS mame (" \
                 "build, " \
@@ -65,6 +70,8 @@
                 "input_tilt DEFAULT 'no' " \
                 "CHECK (input_tilt in ('yes', 'no')), " \
                 "input_players, " \
+                "input_players_alt, " \
+                "input_players_sim, " \
                 "input_buttons, " \
                 "input_coins, " \
                 "driver_status " \
@@ -177,11 +184,37 @@
 #define SQL_CREATE_TABLE_DIPVALUE \
         "CREATE TABLE IF NOT EXISTS dipvalue (" \
                 "game NOT NULL, " \
+                /* new in 0.136 */ "tag, " \
+                /* new in 0.136 */ "mask, " \
                 "dipswitch NOT NULL, " \
                 "name NOT NULL, " \
+                /* new in 0.136 */ "value, " \
                 "default_ DEFAULT 'no' " \
                 "CHECK (default_ in ('yes', 'no')));"
 
+#define SQL_CREATE_TABLE_CONFSETTING \
+        "CREATE TABLE IF NOT EXISTS confsetting (" \
+                "game NOT NULL, " \
+                /* new in 0.136 */ "tag, " \
+                /* new in 0.136 */ "mask, " \
+                "configuration NOT NULL, " \
+                "name NOT NULL, " \
+                /* new in 0.136 */ "value, " \
+                "default_ DEFAULT 'no' " \
+                "CHECK (default_ in ('yes', 'no')));"
+
+#define SQL_CREATE_TABLE_ADJUSTER \
+        "CREATE TABLE IF NOT EXISTS adjuster (" \
+                "game NOT NULL, " \
+                "name NOT NULL, " \
+                "default_ NOT NULL);"
+
+/* The lastplayed table survives database builds. */
+#define SQL_CREATE_TABLE_LASTPLAYED \
+        "CREATE TABLE IF NOT EXISTS lastplayed (" \
+                "name PRIMARY KEY ON CONFLICT REPLACE, " \
+                "timestamp);"
+
 /* The playback table survives database builds. */
 #define SQL_CREATE_TABLE_PLAYBACK \
         "CREATE TABLE IF NOT EXISTS playback (" \
@@ -202,7 +235,9 @@
 #define SQL_CREATE_VIEW_AVAILABLE \
         "CREATE VIEW IF NOT EXISTS available AS " \
                 "SELECT game.*, bios.description AS bios, " \
-                "isfavorite(game.name) AS favorite FROM game " \
+                "isfavorite(game.name) AS favorite, " \
+                "lastplayed.timestamp AS lastplayed " \
+                "FROM game LEFT JOIN lastplayed USING (name) " \
                 "LEFT JOIN (SELECT name, description FROM game WHERE " \
                 "isbios = 'yes') AS bios ON game.romof = bios.name " \
                 "WHERE (romset IN ('good', 'best available') " \
@@ -219,6 +254,8 @@
         "DROP TABLE IF EXISTS display; " \
         "DROP TABLE IF EXISTS control; " \
         "DROP TABLE IF EXISTS dipvalue; " \
+        "DROP TABLE IF EXISTS confsetting; " \
+        "DROP TABLE IF EXISTS adjuster; " \
         "DROP VIEW IF EXISTS available"
 
 #define SQL_INSERT_GAME \
@@ -240,6 +277,8 @@
                 "@input_service, " \
                 "@input_tilt, " \
                 "@input_players, " \
+                "@input_players_alt, " \
+                "@input_players_sim, " \
                 "@input_buttons, " \
                 "@input_coins, " \
                 "@driver_status, " \
@@ -327,8 +366,27 @@
 #define SQL_INSERT_DIPVALUE \
         "INSERT INTO dipvalue VALUES (" \
                 "@game, " \
+                "@tag, " \
+                "@mask, " \
                 "@dipswitch, " \
                 "@name, " \
+                "@value, " \
+                "@default_);"
+
+#define SQL_INSERT_CONFSETTING \
+        "INSERT INTO confsetting VALUES (" \
+                "@game, " \
+                "@tag, " \
+                "@mask, " \
+                "@configuration, " \
+                "@name, " \
+                "@value, " \
+                "@default_);"
+
+#define SQL_INSERT_ADJUSTER \
+        "INSERT INTO adjuster VALUES (" \
+                "@game, " \
+                "@name, " \
                 "@default_);"
 
 typedef struct _ParserData ParserData;
@@ -347,16 +405,22 @@ struct _ParserData
         sqlite3_stmt *insert_display_stmt;
         sqlite3_stmt *insert_control_stmt;
         sqlite3_stmt *insert_dipvalue_stmt;
+        sqlite3_stmt *insert_confsetting_stmt;
+        sqlite3_stmt *insert_adjuster_stmt;
 
         const gchar *element_stack[MAX_ELEMENT_DEPTH];
         guint element_stack_depth;
+        gchar *configuration;
         gchar *dipswitch;
         gchar *game;
+        gchar *mask;
+        gchar *tag;
 };
 
 /* Canonical names of XML elements and attributes */
 static struct
 {
+        const gchar *adjuster;
         const gchar *aspectx;
         const gchar *aspecty;
         const gchar *bios;
@@ -370,6 +434,8 @@ static struct
         const gchar *cocktail;
         const gchar *coins;
         const gchar *color;
+        const gchar *configuration;
+        const gchar *confsetting;
         const gchar *control;
         const gchar *crc;
         const gchar *default_;
@@ -394,6 +460,7 @@ static struct
         const gchar *keydelta;
         const gchar *mame;
         const gchar *manufacturer;
+        const gchar *mask;
         const gchar *maximum;
         const gchar *md5;
         const gchar *merge;
@@ -423,8 +490,10 @@ static struct
         const gchar *sound;
         const gchar *sourcefile;
         const gchar *status;
+        const gchar *tag;
         const gchar *tilt;
         const gchar *type;
+        const gchar *value;
         const gchar *vbend;
         const gchar *vbstart;
         const gchar *vtotal;
@@ -435,6 +504,25 @@ static struct
 
 static sqlite3 *db = NULL;
 
+static void
+db_parser_bind_int (sqlite3_stmt *stmt,
+                    const gchar *param,
+                    gint value)
+{
+        gint index;
+        gint errcode;
+        GError *error = NULL;
+
+        index = sqlite3_bind_parameter_index (stmt, param);
+        errcode = sqlite3_bind_int (stmt, index, value);
+
+        if (errcode != SQLITE_OK)
+        {
+                gva_db_set_error (&error, 0, NULL);
+                gva_error_handle (&error);
+        }
+}
+
 static void
 db_parser_bind_text (sqlite3_stmt *stmt,
                      const gchar *param,
@@ -475,6 +563,32 @@ db_parser_exec_stmt (sqlite3_stmt *stmt,
         return TRUE;
 }
 
+static void
+db_parser_start_element_adjuster (ParserData *data,
+                                  const gchar **attribute_name,
+                                  const gchar **attribute_value,
+                                  GError **error)
+{
+        sqlite3_stmt *stmt = data->insert_adjuster_stmt;
+        gint ii;
+
+        db_parser_bind_text (stmt, "@game", data->game);
+
+        for (ii = 0; attribute_name[ii] != NULL; ii++)
+        {
+                const gchar *param;
+
+                if (attribute_name[ii] == intern.name)
+                        param = "@name";
+                else if (attribute_name[ii] == intern.default_)
+                        param = "@default_";
+                else
+                        continue;
+
+                db_parser_bind_text (stmt, param, attribute_value[ii]);
+        }
+}
+
 static void
 db_parser_start_element_biosset (ParserData *data,
                                  const gchar **attribute_name,
@@ -534,6 +648,65 @@ db_parser_start_element_chip (ParserData *data,
         }
 }
 
+static void
+db_parser_start_element_configuration (ParserData *data,
+                                       const gchar **attribute_name,
+                                       const gchar **attribute_value,
+                                       GError **error)
+{
+        gint ii;
+
+        for (ii = 0; attribute_name[ii] != NULL; ii++)
+        {
+                if (attribute_name[ii] == intern.mask)
+                        data->mask = g_strdup (attribute_value[ii]);
+                if (attribute_name[ii] == intern.name)
+                        data->configuration = g_strdup (attribute_value[ii]);
+                if (attribute_name[ii] == intern.tag)
+                        data->tag = g_strdup (attribute_value[ii]);
+        }
+}
+
+static void
+db_parser_start_element_confsetting (ParserData *data,
+                                     const gchar **attribute_name,
+                                     const gchar **attribute_value,
+                                     GError **error)
+{
+        sqlite3_stmt *stmt = data->insert_confsetting_stmt;
+        gint ii;
+
+        /* Bind default values. */
+        db_parser_bind_text (stmt, "@default_", "no");
+
+        /* XXX Combining the <configuration> and <confsetting> elements
+         *     into one table is biting us now since 0.136 added "tag"
+         *     and "mask" attributes to <configuration>, because we now
+         *     have to duplicate those values in each "confsetting" row.
+         *     We may need to redesign these tables if we ever actually
+         *     use them. */
+        db_parser_bind_text (stmt, "@game", data->game);
+        db_parser_bind_text (stmt, "@tag", data->tag);
+        db_parser_bind_text (stmt, "@mask", data->mask);
+        db_parser_bind_text (stmt, "@configuration", data->configuration);
+
+        for (ii = 0; attribute_name[ii] != NULL; ii++)
+        {
+                const gchar *param;
+
+                if (attribute_name[ii] == intern.name)
+                        param = "@name";
+                else if (attribute_name[ii] == intern.default_)
+                        param = "@default_";
+                else if (attribute_name[ii] == intern.value)
+                        param = "@value";
+                else
+                        continue;
+
+                db_parser_bind_text (stmt, param, attribute_value[ii]);
+        }
+}
+
 static void
 db_parser_start_element_control (ParserData *data,
                                  const gchar **attribute_name,
@@ -580,8 +753,14 @@ db_parser_start_element_dipswitch (ParserData *data,
         gint ii;
 
         for (ii = 0; attribute_name[ii] != NULL; ii++)
+        {
+                if (attribute_name[ii] == intern.mask)
+                        data->mask = g_strdup (attribute_value[ii]);
                 if (attribute_name[ii] == intern.name)
                         data->dipswitch = g_strdup (attribute_value[ii]);
+                if (attribute_name[ii] == intern.tag)
+                        data->tag = g_strdup (attribute_value[ii]);
+        }
 }
 
 static void
@@ -596,7 +775,15 @@ db_parser_start_element_dipvalue (ParserData *data,
         /* Bind default values. */
         db_parser_bind_text (stmt, "@default_", "no");
 
+        /* XXX Combining the <dipswitch> and <dipvalue> elements into
+         *     one table is biting us now since 0.136 added "tag" and
+         *     "mask" attributes to <dipswitch>, because we now have
+         *     to duplicate those values in each "dipvalue" row.  We
+         *     may need to redesign these tables if we ever actually
+         *     use them. */
         db_parser_bind_text (stmt, "@game", data->game);
+        db_parser_bind_text (stmt, "@tag", data->tag);
+        db_parser_bind_text (stmt, "@mask", data->mask);
         db_parser_bind_text (stmt, "@dipswitch", data->dipswitch);
 
         for (ii = 0; attribute_name[ii] != NULL; ii++)
@@ -607,6 +794,8 @@ db_parser_start_element_dipvalue (ParserData *data,
                         param = "@name";
                 else if (attribute_name[ii] == intern.default_)
                         param = "@default_";
+                else if (attribute_name[ii] == intern.value)
+                        param = "@value";
                 else
                         continue;
 
@@ -816,6 +1005,13 @@ db_parser_start_element_input (ParserData *data,
         sqlite3_stmt *stmt = data->insert_game_stmt;
         gint ii;
 
+#ifdef NPLAYERS_FILE
+        gint error_code;
+        gint max_alternating = 0;
+        gint max_simultaneous = 0;
+        GError *local_error = NULL;
+#endif
+
         /* Bind default values. */
         db_parser_bind_text (stmt, "@input_service", "no");
         db_parser_bind_text (stmt, "@input_tilt", "no");
@@ -839,6 +1035,31 @@ db_parser_start_element_input (ParserData *data,
 
                 db_parser_bind_text (stmt, param, attribute_value[ii]);
         }
+
+#ifdef NPLAYERS_FILE
+        /* Lookup players info from the nplayers.ini file. */
+        g_return_if_fail (data->game != NULL);
+        gva_nplayers_lookup (
+                data->game, &max_alternating,
+                &max_simultaneous, &local_error);
+
+        /* Silence "key not found" errors. */
+        error_code = G_KEY_FILE_ERROR_KEY_NOT_FOUND;
+        if (g_error_matches (local_error, G_KEY_FILE_ERROR, error_code))
+                g_clear_error (&local_error);
+
+        db_parser_bind_int (stmt, "@input_players_alt", max_alternating);
+        db_parser_bind_int (stmt, "@input_players_sim", max_simultaneous);
+
+        /* Override "input_players" if we can, because nplayers.ini
+         * seems to be more accurate than MAME's own XML data. */
+        if (max_alternating > 0 || max_simultaneous > 0)
+                db_parser_bind_int (
+                        stmt, "@input_players",
+                        MAX (max_alternating, max_simultaneous));
+        else if (local_error != NULL)
+                g_propagate_error (error, local_error);
+#endif
 }
 
 static void
@@ -1012,7 +1233,11 @@ db_parser_start_element (GMarkupParseContext *context,
         /* Skip unused elements to speed up parsing. */
 
 #if 0
-        if (element_name == intern.biosset)
+        if (element_name == intern.adjuster)
+                db_parser_start_element_adjuster (
+                        data, attribute_name, attribute_value, error);
+
+        else if (element_name == intern.biosset)
                 db_parser_start_element_biosset (
                         data, attribute_name, attribute_value, error);
 
@@ -1020,6 +1245,14 @@ db_parser_start_element (GMarkupParseContext *context,
                 db_parser_start_element_chip (
                         data, attribute_name, attribute_value, error);
 
+        else if (element_name == intern.configuration)
+                db_parser_start_element_configuration (
+                        data, attribute_name, attribute_value, error);
+
+        else if (element_name == intern.confsetting)
+                db_parser_start_element_confsetting (
+                        data, attribute_name, attribute_value, error);
+
         else if (element_name == intern.control)
                 db_parser_start_element_control (
                         data, attribute_name, attribute_value, error);
@@ -1070,10 +1303,30 @@ db_parser_start_element (GMarkupParseContext *context,
 #endif
 }
 
+static void
+db_parser_end_element_configuration (ParserData *data,
+                                     GError **error)
+{
+        g_free (data->tag);
+        data->tag = NULL;
+
+        g_free (data->mask);
+        data->mask = NULL;
+
+        g_free (data->configuration);
+        data->configuration = NULL;
+}
+
 static void
 db_parser_end_element_dipswitch (ParserData *data,
                                  GError **error)
 {
+        g_free (data->tag);
+        data->tag = NULL;
+
+        g_free (data->mask);
+        data->mask = NULL;
+
         g_free (data->dipswitch);
         data->dipswitch = NULL;
 }
@@ -1116,12 +1369,21 @@ db_parser_end_element (GMarkupParseContext *context,
         /* Skip unused elements to speed up parsing. */
 
 #if 0
-        if (element_name == intern.biosset)
+        if (element_name == intern.adjuster)
+                db_parser_exec_stmt (data->insert_adjuster_stmt, error);
+
+        else if (element_name == intern.biosset)
                 db_parser_exec_stmt (data->insert_biosset_stmt, error);
 
         else if (element_name == intern.chip)
                 db_parser_exec_stmt (data->insert_chip_stmt, error);
 
+        else if (element_name == intern.configuration)
+                db_parser_end_element_configuration (data, error);
+
+        else if (element_name == intern.confsetting)
+                db_parser_exec_stmt (data->insert_confsetting_stmt, error);
+
         else if (element_name == intern.control)
                 db_parser_exec_stmt (data->insert_control_stmt, error);
 
@@ -1218,6 +1480,12 @@ db_parser_data_new (GvaProcess *process)
         if (!gva_db_prepare (SQL_INSERT_DIPVALUE, &data->insert_dipvalue_stmt, &error))
                 g_error ("%s", error->message);
 
+        if (!gva_db_prepare (SQL_INSERT_CONFSETTING, &data->insert_confsetting_stmt, &error))
+                g_error ("%s", error->message);
+
+        if (!gva_db_prepare (SQL_INSERT_ADJUSTER, &data->insert_adjuster_stmt, &error))
+                g_error ("%s", error->message);
+
         return data;
 }
 
@@ -1236,9 +1504,14 @@ db_parser_data_free (ParserData *data)
         sqlite3_finalize (data->insert_display_stmt);
         sqlite3_finalize (data->insert_control_stmt);
         sqlite3_finalize (data->insert_dipvalue_stmt);
+        sqlite3_finalize (data->insert_confsetting_stmt);
+        sqlite3_finalize (data->insert_adjuster_stmt);
 
+        g_free (data->configuration);
         g_free (data->dipswitch);
         g_free (data->game);
+        g_free (data->mask);
+        g_free (data->tag);
 
         g_slice_free (ParserData, data);
 }
@@ -1300,14 +1573,17 @@ db_create_tables (GError **error)
 {
         return gva_db_execute (SQL_CREATE_TABLE_MAME, error)
                 && gva_db_execute (SQL_CREATE_TABLE_GAME, error)
+                && gva_db_execute (SQL_CREATE_TABLE_ADJUSTER, error)
                 && gva_db_execute (SQL_CREATE_TABLE_BIOSSET, error)
                 && gva_db_execute (SQL_CREATE_TABLE_ROM, error)
                 && gva_db_execute (SQL_CREATE_TABLE_DISK, error)
                 && gva_db_execute (SQL_CREATE_TABLE_SAMPLE, error)
                 && gva_db_execute (SQL_CREATE_TABLE_CHIP, error)
+                && gva_db_execute (SQL_CREATE_TABLE_CONFSETTING, error)
                 && gva_db_execute (SQL_CREATE_TABLE_DISPLAY, error)
                 && gva_db_execute (SQL_CREATE_TABLE_CONTROL, error)
                 && gva_db_execute (SQL_CREATE_TABLE_DIPVALUE, error)
+                && gva_db_execute (SQL_CREATE_TABLE_LASTPLAYED, error)
                 && gva_db_execute (SQL_CREATE_TABLE_PLAYBACK, error)
                 && gva_db_execute (SQL_CREATE_TABLE_WINDOW, error)
                 && gva_db_execute (SQL_CREATE_VIEW_AVAILABLE, error);
@@ -1437,79 +1713,85 @@ gva_db_build (GError **error)
         g_return_val_if_fail (db != NULL, NULL);
 
         /* Initialize the list of canonical names. */
-        intern.aspectx      = g_intern_static_string ("aspectx");
-        intern.aspecty      = g_intern_static_string ("aspecty");
-        intern.bios         = g_intern_static_string ("bios");
-        intern.biosset      = g_intern_static_string ("biosset");
-        intern.build        = g_intern_static_string ("build");
-        intern.buttons      = g_intern_static_string ("buttons");
-        intern.channels     = g_intern_static_string ("channels");
-        intern.chip         = g_intern_static_string ("chip");
-        intern.clock        = g_intern_static_string ("clock");
-        intern.cloneof      = g_intern_static_string ("cloneof");
-        intern.cocktail     = g_intern_static_string ("cocktail");
-        intern.coins        = g_intern_static_string ("coins");
-        intern.color        = g_intern_static_string ("color");
-        intern.control      = g_intern_static_string ("control");
-        intern.crc          = g_intern_static_string ("crc");
-        intern.default_     = g_intern_static_string ("default_");
-        intern.description  = g_intern_static_string ("description");
-        intern.dipswitch    = g_intern_static_string ("dipswitch");
-        intern.dipvalue     = g_intern_static_string ("dipvalue");
-        intern.disk         = g_intern_static_string ("disk");
-        intern.display      = g_intern_static_string ("display");
-        intern.dispose      = g_intern_static_string ("dispose");
-        intern.driver       = g_intern_static_string ("driver");
-        intern.emulation    = g_intern_static_string ("emulation");
-        intern.flipx        = g_intern_static_string ("flipx");
-        intern.game         = g_intern_static_string ("game");
-        intern.graphic      = g_intern_static_string ("graphic");
-        intern.hbend        = g_intern_static_string ("hbend");
-        intern.hbstart      = g_intern_static_string ("hbstart");
-        intern.height       = g_intern_static_string ("height");
-        intern.htotal       = g_intern_static_string ("htotal");
-        intern.index_       = g_intern_static_string ("index_");
-        intern.input        = g_intern_static_string ("input");
-        intern.isbios       = g_intern_static_string ("isbios");
-        intern.keydelta     = g_intern_static_string ("keydelta");
-        intern.mame         = g_intern_static_string ("mame");
-        intern.manufacturer = g_intern_static_string ("manufacturer");
-        intern.maximum      = g_intern_static_string ("maximum");
-        intern.md5          = g_intern_static_string ("md5");
-        intern.merge        = g_intern_static_string ("merge");
-        intern.minimum      = g_intern_static_string ("minimum");
-        intern.name         = g_intern_static_string ("name");
-        intern.offset       = g_intern_static_string ("offset");
-        intern.orientation  = g_intern_static_string ("orientation");
-        intern.palettesize  = g_intern_static_string ("palettesize");
-        intern.pixclock     = g_intern_static_string ("pixclock");
-        intern.players      = g_intern_static_string ("players");
-        intern.protection   = g_intern_static_string ("protection");
-        intern.refresh      = g_intern_static_string ("refresh");
-        intern.region       = g_intern_static_string ("region");
-        intern.reverse      = g_intern_static_string ("reverse");
-        intern.rom          = g_intern_static_string ("rom");
-        intern.romof        = g_intern_static_string ("romof");
-        intern.rotate       = g_intern_static_string ("rotate");
-        intern.runnable     = g_intern_static_string ("runnable");
-        intern.sample       = g_intern_static_string ("sample");
-        intern.sampleof     = g_intern_static_string ("sampleof");
-        intern.savestate    = g_intern_static_string ("savestate");
-        intern.screen       = g_intern_static_string ("screen");
-        intern.sensitivity  = g_intern_static_string ("sensitivity");
-        intern.service      = g_intern_static_string ("service");
-        intern.sha1         = g_intern_static_string ("sha1");
-        intern.size         = g_intern_static_string ("size");
-        intern.sound        = g_intern_static_string ("sound");
-        intern.sourcefile   = g_intern_static_string ("sourcefile");
-        intern.status       = g_intern_static_string ("status");
-        intern.tilt         = g_intern_static_string ("tilt");
-        intern.type         = g_intern_static_string ("type");
-        intern.vbend        = g_intern_static_string ("vbend");
-        intern.vbstart      = g_intern_static_string ("vbstart");
-        intern.vtotal       = g_intern_static_string ("vtotal");
-        intern.width        = g_intern_static_string ("width");
-        intern.year         = g_intern_static_string ("year");
+        intern.adjuster      = g_intern_static_string ("adjuster");
+        intern.aspectx       = g_intern_static_string ("aspectx");
+        intern.aspecty       = g_intern_static_string ("aspecty");
+        intern.bios          = g_intern_static_string ("bios");
+        intern.biosset       = g_intern_static_string ("biosset");
+        intern.build         = g_intern_static_string ("build");
+        intern.buttons       = g_intern_static_string ("buttons");
+        intern.channels      = g_intern_static_string ("channels");
+        intern.chip          = g_intern_static_string ("chip");
+        intern.clock         = g_intern_static_string ("clock");
+        intern.cloneof       = g_intern_static_string ("cloneof");
+        intern.cocktail      = g_intern_static_string ("cocktail");
+        intern.coins         = g_intern_static_string ("coins");
+        intern.color         = g_intern_static_string ("color");
+        intern.configuration = g_intern_static_string ("configuration");
+        intern.confsetting   = g_intern_static_string ("confsetting");
+        intern.control       = g_intern_static_string ("control");
+        intern.crc           = g_intern_static_string ("crc");
+        intern.default_      = g_intern_static_string ("default_");
+        intern.description   = g_intern_static_string ("description");
+        intern.dipswitch     = g_intern_static_string ("dipswitch");
+        intern.dipvalue      = g_intern_static_string ("dipvalue");
+        intern.disk          = g_intern_static_string ("disk");
+        intern.display       = g_intern_static_string ("display");
+        intern.dispose       = g_intern_static_string ("dispose");
+        intern.driver        = g_intern_static_string ("driver");
+        intern.emulation     = g_intern_static_string ("emulation");
+        intern.flipx         = g_intern_static_string ("flipx");
+        intern.game          = g_intern_static_string ("game");
+        intern.graphic       = g_intern_static_string ("graphic");
+        intern.hbend         = g_intern_static_string ("hbend");
+        intern.hbstart       = g_intern_static_string ("hbstart");
+        intern.height        = g_intern_static_string ("height");
+        intern.htotal        = g_intern_static_string ("htotal");
+        intern.index_        = g_intern_static_string ("index_");
+        intern.input         = g_intern_static_string ("input");
+        intern.isbios        = g_intern_static_string ("isbios");
+        intern.keydelta      = g_intern_static_string ("keydelta");
+        intern.mame          = g_intern_static_string ("mame");
+        intern.manufacturer  = g_intern_static_string ("manufacturer");
+        intern.mask          = g_intern_static_string ("mask");
+        intern.maximum       = g_intern_static_string ("maximum");
+        intern.md5           = g_intern_static_string ("md5");
+        intern.merge         = g_intern_static_string ("merge");
+        intern.minimum       = g_intern_static_string ("minimum");
+        intern.name          = g_intern_static_string ("name");
+        intern.offset        = g_intern_static_string ("offset");
+        intern.orientation   = g_intern_static_string ("orientation");
+        intern.palettesize   = g_intern_static_string ("palettesize");
+        intern.pixclock      = g_intern_static_string ("pixclock");
+        intern.players       = g_intern_static_string ("players");
+        intern.protection    = g_intern_static_string ("protection");
+        intern.refresh       = g_intern_static_string ("refresh");
+        intern.region        = g_intern_static_string ("region");
+        intern.reverse       = g_intern_static_string ("reverse");
+        intern.rom           = g_intern_static_string ("rom");
+        intern.romof         = g_intern_static_string ("romof");
+        intern.rotate        = g_intern_static_string ("rotate");
+        intern.runnable      = g_intern_static_string ("runnable");
+        intern.sample        = g_intern_static_string ("sample");
+        intern.sampleof      = g_intern_static_string ("sampleof");
+        intern.savestate     = g_intern_static_string ("savestate");
+        intern.screen        = g_intern_static_string ("screen");
+        intern.sensitivity   = g_intern_static_string ("sensitivity");
+        intern.service       = g_intern_static_string ("service");
+        intern.sha1          = g_intern_static_string ("sha1");
+        intern.size          = g_intern_static_string ("size");
+        intern.sound         = g_intern_static_string ("sound");
+        intern.sourcefile    = g_intern_static_string ("sourcefile");
+        intern.status        = g_intern_static_string ("status");
+        intern.tag           = g_intern_static_string ("tag");
+        intern.tilt          = g_intern_static_string ("tilt");
+        intern.type          = g_intern_static_string ("type");
+        intern.value         = g_intern_static_string ("value");
+        intern.vbend         = g_intern_static_string ("vbend");
+        intern.vbstart       = g_intern_static_string ("vbstart");
+        intern.vtotal        = g_intern_static_string ("vtotal");
+        intern.width         = g_intern_static_string ("width");
+        intern.year          = g_intern_static_string ("year");
 
         if (!gva_db_reset (error))
                 return NULL;
