@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -150,6 +150,10 @@ gva_main_init (void)
                 gva_ui_get_managed_widget ("/main-menu"),
                 FALSE, FALSE, 0);
 
+        gtk_box_reorder_child (
+                GTK_BOX (GVA_WIDGET_MAIN_VBOX),
+                gva_ui_get_managed_widget ("/main-menu"), 0);
+
         gtk_action_connect_proxy (
                 GVA_ACTION_VIEW_AVAILABLE,
                 GVA_WIDGET_MAIN_VIEW_BUTTON_0);
@@ -170,6 +174,14 @@ gva_main_init (void)
                 GVA_ACTION_START,
                 GVA_WIDGET_MAIN_START_GAME_BUTTON);
 
+        gtk_widget_set_sensitive (
+                GVA_WIDGET_MAIN_MUTE_BUTTON,
+                gva_mame_supports_sound ());
+
+        gconf_bridge_bind_property (
+                gconf_bridge_get (), GVA_GCONF_SOUND_MUTED,
+                G_OBJECT (GVA_WIDGET_MAIN_MUTE_BUTTON), "muted");
+
         gconf_bridge_bind_window (
                 gconf_bridge_get (), GVA_GCONF_WINDOW_PREFIX,
                 GTK_WINDOW (GVA_WIDGET_MAIN_WINDOW), TRUE, FALSE);
@@ -179,7 +191,7 @@ gva_main_init (void)
         gtk_entry_set_text (GTK_ENTRY (GVA_WIDGET_MAIN_SEARCH_ENTRY), text);
         g_free (text);
 
-#ifndef WITH_GNOME
+#if !GTK_CHECK_VERSION(2,14,0) && !defined HAVE_GNOME
         /* Requires that we link against libgnome. */
         gtk_action_set_sensitive (GVA_ACTION_CONTENTS, FALSE);
 #endif
@@ -344,7 +356,7 @@ gva_main_init_search_completion (GError **error)
         gint errcode;
 
 #if GTK_CHECK_VERSION(2,13,1)
-	GList *list;
+        GList *list;
 #endif
 
         if (!gva_db_prepare (SQL_COMPLETION_LIST, &stmt, error))
@@ -441,10 +453,6 @@ gva_main_connect_proxy_cb (GtkUIManager *manager,
         /* Show GtkMenuItem tooltips in the statusbar. */
         if (GTK_IS_MENU_ITEM (proxy))
         {
-                GtkStatusbar *statusbar;
-
-                statusbar = GTK_STATUSBAR (GVA_WIDGET_MAIN_STATUSBAR);
-
                 g_signal_connect (
                         proxy, "select",
                         G_CALLBACK (main_menu_item_select_cb), action);
